{ *** A simple MODE/STTY command for text files.
  ***
  *** Usage:    MODE baud,databits,parity,stopbits
  *** Examples: MODE 19200,8,N,1
  ***           MODE 38400,5,E,15
  ***           MODE
  *** To set 1.5 stop bits enter '15'
  *** If no arguments are given, the current settings are output.
  ***
  *** Martin Hepperle, September 2018
  *** }
Program Stty;

Const
  I2C_UART_BASE = $50; { base port address of I2C register set }
  I2C_UART_THR  = $0;  { transmit hold register }
  I2C_UART_LSR  = $5;  { line status register }

Var
  i,j,p:    Integer;
  Buffer:   String[128];
  Param:    String[20];
  Baud:     Real;
  Code:     Integer;

{ ------------------------------------- }

Procedure DumpRegister ( Register: Integer );
Var
  Status: Byte;
  i:      Integer;
  Buffer: String[20];
Begin
  Status := Port[I2C_UART_BASE + Register];
  Write ( 'UART Register[', Register, '] = ');
  WriteLn ( Status );

  WriteLn ( ' 7 6 5 4 3 2 1 0' );
  Buffer := '';
  For  i := 1 To 8 Do
  Begin
    If ( (Status and 1) = 1 ) Then
      Buffer := ' 1' + Buffer
    Else
      Buffer := ' 0' + Buffer;
    Status := Status Shr 1;
  End;
  WriteLn ( Buffer );
End;

{ ------------------------------------- }

Procedure SetBaudRate ( BaudRate: Real );
  { --- setup of I2C-UART baud rate }
  { BaudRate: 50 ... 38400+ }
Var
  R03:     Byte;
  Divisor: Integer;

Begin

  { -- save current settings }
  R03 := Port[I2C_UART_BASE + $03];
  { --- allow access special registers 0 and 1 }
  Port[I2C_UART_BASE + $03] := 128;

  { --- set divisor for baud rate (for 14.7456 MHz crystal) }
  Divisor := Trunc( 9216 / (BaudRate/100) );
  Port[I2C_UART_BASE + $00] := Divisor and $FF;
  Port[I2C_UART_BASE + $01] := (Divisor shr 8) and $FF;

  { --- disallow access special registers 0 and 1 }
  Port[I2C_UART_BASE + $03] := R03;

End;

{ ------------------------------------- }

Function GetBaudRate: Real;
  { --- return the I2C-UART baud rate }
Var
  R03:      Byte;

Begin

  { -- save current settings }
  R03 := Port[I2C_UART_BASE + $03];

  { --- allow access special registers 0 and 1 }
  Port[I2C_UART_BASE + $03] := 128;

  { --- set divisor for baud rate (for 14.7456 MHz crystal) }
  GetBaudRate := 921600.0 / ( Port[I2C_UART_BASE + $00] * 1.0 +
                              Port[I2C_UART_BASE + $01] * 256 );

  { --- disallow access special registers 0 and 1 }
  Port[I2C_UART_BASE + $03] := R03;

End;

{ ------------------------------------- }

Procedure SetBitCount ( BitCount: Integer );
  { --- setup of I2C-UART bit count }
  { BitCount: 5,6,7,8 }

Var
  R03: Byte;

Begin

  { -- get current settings }
  R03 := Port[I2C_UART_BASE + $03];

  R03 := R03 and $FC; { clear bits 0 and 1: == default of 5 bit }

  Case BitCount Of
    5: Begin End;                  { 5 bit, alread set }
    6: Begin R03 := R03 or 1; End; { 6 bit }
    7: Begin R03 := R03 or 2; End; { 7 bit }
    8: Begin R03 := R03 or 3; End; { 8 bit }
    Else Begin WriteLn('Unknown bit count: ',BitCount); End;
  End;

  Port[I2C_UART_BASE + $03] := R03;

End;

{ ------------------------------------- }

Function GetBitCount: Integer;
  { --- return the I2C-UART bit count }
  { BitCount: 5,6,7,8 }

Var
  R03: Byte;

Begin

  { -- get current settings }
  R03 := Port[I2C_UART_BASE + $03];

  R03 := R03 and $03; { mask bits 0 and 1 }

  Case R03 Of
    0: Begin GetBitCount := 5; End; { 5 bit }
    1: Begin GetBitCount := 6; End; { 6 bit }
    2: Begin GetBitCount := 7; End; { 7 bit }
    3: Begin GetBitCount := 8; End; { 8 bit }
  End;

End;

{ ------------------------------------- }

Procedure SetParity ( Parity: Char );
  { --- setup of I2C-UART Parity bits }
  { Parity: 'n', 'e', 'o', '0', '1' }
Var
  R03: Byte;

Begin

  { -- get current settings }
  R03 := Port[I2C_UART_BASE + $03];

  R03 := R03 and $C7; { clear bits 3,4 and, 5 }

  Parity := UpCase(Parity);
  Case Parity Of
    'N': Begin R03 := R03 or $00; End; { xx0--- none }
    'E': Begin R03 := R03 or $18; End; { 011--- even }
    'O': Begin R03 := R03 or $08; End; { 001--- odd }
    '1': Begin R03 := R03 or $28; End; { 101--- forced 1 }
    '0': Begin R03 := R03 or $38; End; { 111--- forced 0 }
    Else Begin WriteLn('Unknown parity: ',Parity); End;
  End;

  Port[I2C_UART_BASE + $03] := R03;

End;

{ ------------------------------------- }

Function GetParity: Char;
  { --- return the I2C-UART Parity bits }
  { Parity: 'n', 'e', 'o', '0', '1' }
Var
  R03: Byte;

Begin

  { -- get current settings }
  R03 := Port[I2C_UART_BASE + $03];

  R03 := (R03 and $38) shr 3; { mask bits 3,4 and, 5 }

  If (R03 and $01) = 0 Then
  Begin
    GetParity := 'N';                 { xx0--- none }
  End
  Else
  Begin
    Case R03 Of
      1: Begin GetParity := 'O'; End; { 001--- odd }
      3: Begin GetParity := 'E'; End; { 011--- even }
      5: Begin GetParity := '1'; End; { 101--- even }
      7: Begin GetParity := '0'; End; { 111--- even }
    End
  End;

End;

{ ------------------------------------- }

Procedure SetStopBits ( StopBits: Byte );
  { --- setup of I2C-UART Stop bits }
  { StopBits: 1,15,2 (must be compatible to BitCount) }
Var
  R03: Byte;

Begin

  { -- get current settings }
  R03 := Port[I2C_UART_BASE + $03];

  R03 := R03 and $FB; { clear bit 2 }

  Case StopBits Of
    1:    Begin R03 := R03 or $00; End; { 0-- 1   for 5,6,7,8 bits }
    2:    Begin R03 := R03 or $04; End; { 1-- 1.5 for 5 bits }
    15:   Begin R03 := R03 or $04; End; { 1-- 2   for 6,7,8 bits }
    Else Begin WriteLn('Unknown stop bit count: ',StopBits); End;
  End;

  Port[I2C_UART_BASE + $03] := R03;

End;

{ ------------------------------------- }

Function GetStopBits: Byte;
  { --- return the I2C-UART Stop bits }
  { StopBits: 1,15,2 (must be compatible to BitCount) }
Var
  R03: Byte;

Begin

  { -- get current settings }
  R03 := Port[I2C_UART_BASE + $03];

  R03 := R03 and $04; { mask bit 2 }

  Case R03 Of
    0:   Begin GetStopBits := 1; End;  { 0-- 1   for 5,6,7,8 bits }
    4:   Begin
           If GetBitCount = 5 Then GetStopBits := 15
                              Else GetStopBits := 2;
         End;
  End;

End;

{ ------------------------------------- }

Begin

  If ParamCount > 0 Then
  Begin
    Buffer := ParamStr(1) + ',';
    p := 1;
    i := 1;
    For j:=1 To Length(Buffer) Do
    Begin
      If Buffer[j] = ',' Then
      Begin
        Param := Copy(ParamStr(1),i,j-i);
        Case p Of
          1: Begin Val(Param,Baud,Code); SetBaudRate(Baud); End;
          2: Begin Val(Param,i,Code);    SetBitCount(i);    End;
          3: Begin SetParity(Param[1]);                     End;
          4: Begin Val(Param,i,Code);    SetStopBits(i);    End;
        End;
        p := p+1;
        i := j+1;
      End;
    End;
  End
  Else
  Begin
     WriteLn ( 'Speed    = ', GetBaudRate6:0 );
     WriteLn ( 'BitCount = ', GetBitCount );
     WriteLn ( 'Parity   = ', GetParity );
     WriteLn ( 'StopBits = ', GetStopBits );
  End;
End.

{ ------------------------------------- }
